%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Main funtion
%     rgb_dem = demosaick(rgb, pattern, sigma)
%
%      Input
%       - rgb		: full RGB image or mosaic data
%       - pattern	: mosaic pattern 
%                       default : 'grbg'
%                       others  : 'rggb','gbrg','bggr'	
%       - sigma		: standard deviation of gaussian filter(default : 1)
%                       * For Kodak image data set, 1e8 works well.
%
%      Output 
%       - rgb_dem   : result image
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  rgb_dem = demosaick(rgb, pattern, sigma)

% mosaic and mask
[mosaic mask] = mosaic_bayer(rgb, pattern);

% green interpolation
green = green_interpolation(mosaic, mask, pattern, sigma); 

% parameters for guided upsampling
h = 5;
v = 5;
eps = 0;
% Red and Blue demosaicking
red = red_interpolation(green, mosaic, mask, h, v, eps);
blue = blue_interpolation(green, mosaic, mask, h, v, eps);

% result image
rgb_dem(:,:,1) = red;
rgb_dem(:,:,2) = green;
rgb_dem(:,:,3) = blue;

end
